@@ -15,8 +15,8 @@ see HTML::Robot::Scrapper::Parser::Default
 =cut
 
 sub parse_xml {
-    my ($self, $robot, $content ) = @_;
-    $content = $robot->encoding->safe_encode( $robot->useragent->headers , $content );
+    my ($self, $content ) = @_;
+    $content = $self->robot->encoding->safe_encode( $content );
     my $xml = XML::XPath->new( xml => $content );
     $self->xml( $xml );
 }
